package ru.ulstu.is.sbapp.socialNetwork.controller;

import org.springframework.web.bind.annotation.*;
import ru.ulstu.is.sbapp.WebConfiguration;
import ru.ulstu.is.sbapp.socialNetwork.dto.UserDTO;
import ru.ulstu.is.sbapp.socialNetwork.models.UserModel;
import ru.ulstu.is.sbapp.socialNetwork.services.UserService;


import javax.validation.Valid;
import java.util.List;


@RestController
@RequestMapping(WebConfiguration.REST_API + "/users")
public class UserController {
    private final UserService userService;

    public UserController(UserService userService){
        this.userService = userService;
    }

    @GetMapping("/{id}")
    public UserDTO getUser(@PathVariable Long id) {
        return new UserDTO(userService.findUser(id));
    }

    @GetMapping("")
    public List<UserDTO> getUsers() {
        return userService.findAllUsers().stream().map(UserDTO::new).toList();
    }

    @PostMapping("")
    public UserDTO createUser(@RequestBody @Valid UserDTO user) {
        UserModel result = userService.addUser(user.getName(), user.getCity());
        userService.updateCommunities(result.getId(), user.getCommunity());
        return new UserDTO(userService.updateMusics(result.getId(), user.getMusic()));
    }

    @PatchMapping("/{id}")
    public UserDTO updateUser(@PathVariable Long id,
                              @RequestBody @Valid UserDTO user) {
        UserModel result = userService.updateUser(id, user.getName());
        userService.updateCommunities(result.getId(), user.getCommunity());
        return new UserDTO(userService.updateMusics(result.getId(), user.getMusic()));
    }

    @PatchMapping("/add_music/{id}")
    public UserDTO addMusic(@PathVariable Long id, @RequestParam Long music_id) {
        return new UserDTO(userService.addMusic(id, music_id));
    }

    @PatchMapping("/add_community/{id}")
    public UserDTO addCommunity(@PathVariable Long id, @RequestParam Long community_id) {
        return new UserDTO(userService.addCommunity(id, community_id));
    }
    @GetMapping("/search")
    public List<UserDTO> searchUsers(@RequestParam("city") String city) {
        List<UserModel> users = userService.findUserByCity(city);
        return users.stream().map(UserDTO::new).toList();
    }

    @DeleteMapping("/{id}")
    public UserDTO deleteUser(@PathVariable Long id) {
        return new UserDTO(userService.deleteUser(id));
    }

}
